@@ -22,16 +22,26 @@ class Meter {
     uint64_t gold;
     uint64_t predicted;
     uint64_t predictedGold;
+    mutable std::vector<std::pair<real, real>> scoreVsTrue;
 
     Metrics() : gold(0), predicted(0), predictedGold(0) {}
 
     double precision() const {
+      if (predicted == 0) {
+        return std::numeric_limits<double>::quiet_NaN();
+      }
       return predictedGold / double(predicted);
     }
     double recall() const {
+      if (gold == 0) {
+        return std::numeric_limits<double>::quiet_NaN();
+      }
       return predictedGold / double(gold);
     }
     double f1Score() const {
+      if (predicted + gold == 0) {
+        return std::numeric_limits<double>::quiet_NaN();
+      }
       return 2 * predictedGold / double(predicted + gold);
     }
   };
@@ -46,6 +56,7 @@ class Meter {
   double f1Score(int32_t);
   double precision() const;
   double recall() const;
+  double f1Score() const;
   uint64_t nexamples() const {
     return nexamples_;
   }
